"""

    frosch - Better runtime errors

    Patrick Haller
    betterthannothing.blog
    patrickhaller40@googlemail.com

    License MIT

"""

from .frosch import hook, print_exception

__all__ = ["hook", "print_exception"]
